/*
 * wisc_util.c
 *
 * Utilities for the Wisconsin Benchmark
 *
 * Implemented by David DeWitt and Michael Franklin
 * 
 * Copyright (c) 1991 David DeWitt and Michael Franklin
 *
 */

#include <stdio.h>
#include <sybfront.h>
#include <sybdb.h>
#include "sybdbex.h"
#include <sys/time.h>
#include <sys/wait.h>
#define TMPSTRLEN  256
#include "wiscbench.h"

LOGINREC        *login;

/************************************************************************
 *									* 
 *  time1000	 							*
 *									* 
 * 	get the time in .001 seconds units 				*
 *									* 
 ************************************************************************/

time1000 ()
{
    struct timeval tv;
    struct timezone tz;
    
    gettimeofday(&tv,&tz);
    
    return (tv.tv_sec * 1000 + tv.tv_usec / 1000);
}	
/************************************************************************
 *                                                                      *
 *      setup_session							*
 *                                                                      *
 *         Initalize locking mode and do other DBMS specific things 	*
 *         	prior to running a set of queries			*
 *                                                                      *
 ************************************************************************/
setup_session(show_plan)
	int 	show_plan;
{
    printf("should be setting course granule locks\n");

    /* print query plans if requested */
    if (show_plan) {
	printf("show plan not implemented\n");
    }

    printf("should be turning off journaling\n");

    /* make sure that journaling is turned OFF */
}


/************************************************************************
 *                                                                      *
 *      get_row_count							*
 *                                                                      *
 *         Return a count of the # of rows affected by the last query   *
 *         NOTE: This is Ingres specific, it must be changed based on   *
 *		the method of accessing the SQLCA used by your DBMS.    *
 *                                                                      *
 ************************************************************************/
int
get_row_count(dbproc)
    DBPROCESS        *dbproc;
{
	int rowCount;
	rowCount = dbcount(dbproc);
	return (rowCount);
}

/************************************************************************
 *                                                                      *
 *      create_table                                                    *
 *	  creates a standard wiscbench benchmark table
 *                                                                      *
 *                                                                      *
 ************************************************************************/
create_table(dbproc, tableName, tableNo)
    DBPROCESS        *dbproc;
    char	     *tableName;   /* name of table */
    int              tableNo;	/* which type of table to create */ 
{
    char	query[1000];

    char	type1table[]= "create table %s(unique1D int, unique2D int, twoD int, fourD int, tenD int, twentyD int, onePercentD int, tenPercentD int, twentyPercentD int, fiftyPercentD int, unique3D int, evenOnePercentD int, oddOnePercentD int, stringu1D char(52), stringu2D char(52), string4D char(52))";

    char	type2table[]= "create table %s(unique1E int, unique2E int, twoE int, fourE int, tenE int, twentyE int, onePercentE int, tenPercentE int, twentyPercentE int, fiftyPercentE int, unique3E int, evenOnePercentE int, oddOnePercentE int, stringu1E char(52), stringu2E char(52), string4E char(52))";

    if (tableNo == 1) 
    {
    	sprintf(query,type1table,tableName);
    	dbcmd(dbproc,query);
    	dbsqlexec(dbproc);
    	while (dbresults(dbproc) != NO_MORE_RESULTS) continue;
    }
    else if (tableNo == 2) 
    {
    	sprintf(query,type2table,tableName);
    	dbcmd(dbproc,query);
    	dbsqlexec(dbproc);
    	while (dbresults(dbproc) != NO_MORE_RESULTS) continue;
    }
    else fprintf(stderr,"Error: Bad table number given to create_sel_temp\n");
    return (1);
}


/************************************************************************
 *                                                                      *
 *      create_join_temp                                                *
 *                                                                      *
 *          create temporary relation for join queries                  *
 *                                                                      *
 ************************************************************************/
create_join_temp(dbproc,tableName)
    DBPROCESS        *dbproc;
    char	     *tableName;
{
    char	query[1000];

    char	joinTable[]= "create table %s(unique1D int, unique2D int, twoD int, fourD int, tenD int, twentyD int, onePercentD int, tenPercentD int, twentyPercentD int, fiftyPercentD int, unique3D int, evenOnePercentD int, oddOnePercentD int, stringu1D char(52), stringu2D char(52), string4D char(52), unique1E int, unique2E int, twoE int, fourE int, tenE int, twentyE int, onePercentE int, tenPercentE int, twentyPercentE int, fiftyPercentE int, unique3E int, evenOnePercentE int, oddOnePercentE int, stringu1E char(52), stringu2E char(52), string4E char(52))";

    sprintf(query,joinTable,tableName);
    dbcmd(dbproc,query);
    dbsqlexec(dbproc);
    while (dbresults(dbproc) != NO_MORE_RESULTS) continue;
    return (1);
}

/************************************************************************
 *                                                                      *
 *      create_proj1_temp                                               *
 *                                                                      *
 *          create temporary relation for 1% projection queries         *
 *                                                                      *
 ************************************************************************/
create_proj1_temp(dbproc, table_no)
    DBPROCESS        *dbproc;
    int table_no;	/* which table to create */ 
{
    char	type1Table[] = "create table TMP(twoD int, fourD int, tenD int, twentyD int, onePercentD int, string4D char(52))";

    char	type2Table[] = "create table TMP(twoE int, fourE int, tenE int, twentyE int, onePercentE int, string4E char(52))";

    if (table_no == 1) 
    {
        dbcmd(dbproc,type1Table);
        dbsqlexec(dbproc);
        while (dbresults(dbproc) != NO_MORE_RESULTS) continue;
    }
    else if (table_no == 2) 
    {
        dbcmd(dbproc,type2Table);
        dbsqlexec(dbproc);
        while (dbresults(dbproc) != NO_MORE_RESULTS) continue;
    }
    else {
        fprintf(stderr,"Error: Bad table number given to create_proj1_temp\n");
    }
    return (1);
}

/************************************************************************
 *                                                                      *
 *      create_proj100_temp                                             *
 *                                                                      *
 *          create temporary relation for %100 projectction queries     *
 *                                                                      *
 ************************************************************************/
create_proj100_temp(dbproc, table_no)
    DBPROCESS        *dbproc;
    int table_no;	/* which table to create */ 
{
    char type1Table[] = "create table TMP(twoD int, fourD int, tenD int, twentyD int, onePercentD int, tenPercentD int, twentyPercentD int, fiftyPercentD int, unique3D int, evenOnePercentD int, oddOnePercentD int, stringu1D char(52), stringu2D char(52), string4D char(52))";

    char type2Table[] = "create table TMP(twoE int, fourE int, tenE int, twentyE int, onePercentE int, tenPercentE int, twentyPercentE int, fiftyPercentE int, unique3E int, evenOnePercentE int, oddOnePercentE int, stringu1E char(52), stringu2E char(52), string4E char(52))"; 

    if (table_no == 1) {
        dbcmd(dbproc,type1Table);
    	dbsqlexec(dbproc);
    	while (dbresults(dbproc) != NO_MORE_RESULTS) continue;
    }
    else if (table_no == 2) {
        dbcmd(dbproc,type2Table);
    	dbsqlexec(dbproc);
    	while (dbresults(dbproc) != NO_MORE_RESULTS) continue;
    }
    else 
      fprintf(stderr,"Error: Bad table number given to create_proj100_temp\n");
    return (1);
}


/************************************************************************
 *                                                                      *
 *      create_agg_temp                                                 *
 *                                                                      *
 *          create temporary relation for aggregate queries             *
 *                                                                      *
 ************************************************************************/
create_agg_temp(dbproc)
    DBPROCESS        *dbproc;
{

    dbcmd(dbproc,"create table TMP(");
    dbcmd(dbproc,"uniqueval int)"); 
    dbsqlexec(dbproc);
    while (dbresults(dbproc) != NO_MORE_RESULTS) continue;

    return (1);
}

/************************************************************************
 *                                                                      *
 *      drop_table                                                       *
 *                                                                      *
 *         drop a relation
 *                                                                      *
 ************************************************************************/
drop_table(dbproc,relname)
    DBPROCESS        *dbproc;
    char		*relname;
{
    char query[100];

    sprintf(query,"drop table %s",relname);
    dbcmd(dbproc,query);
    dbsqlexec(dbproc);
    while (dbresults(dbproc) != NO_MORE_RESULTS) continue;
    return (1);
}


/************************************************************************
 *									* 
 *  print_trial  							*
 *									* 
 *      Print the timing info for a trial				*	
 *									* 
 ************************************************************************/

 print_trial(numtrial, rowcount, totaltime)
    int		numtrial, rowcount, totaltime;
{
    fprintf(stderr,"\tTrial# %d, #Rows: %d, Elapsed time: %d.%3.3d Seconds\n",
	numtrial, rowcount, (totaltime)/1000, (totaltime)%1000);
    return (1);
}

/************************************************************************
 *									* 
 *  print_result  							*
 *									* 
 *      Print the timing info for a query run				*	
 *									* 
 ************************************************************************/

 print_result(expname, numtrials, rowcount, totaltime)
    char	*expname;
    int		numtrials, rowcount, totaltime;
{
    int avgtime;

    avgtime = totaltime / numtrials;
    fprintf(stderr,"\n%s: Number of Trials: %d\n", expname,numtrials);
    fprintf(stderr,"\tTotal Row count: %d,  Avg. Elapsed time: %d.%3.3d Seconds\n",
	    rowcount, (avgtime)/1000, (avgtime)%1000);
    
    return (1);
}


/************************************************************************
 *									* 
 *  print_tuple  							*
 *									* 
 *      Print a tuple from a Wisconsin Benchmark relation		*	
 *									* 
 ************************************************************************/

print_tuple(tupPtr)
   BENCHTUP 	*tupPtr;
{
    printf("%d %d %d %d %d %d %d %d %d %d %d %d %d %45s %45s %45s\n",
        tupPtr->unique1, tupPtr->unique2, tupPtr->two,
        tupPtr->four, tupPtr->ten, tupPtr->twenty, tupPtr->onePercent,
        tupPtr->tenPercent, tupPtr->twentyPercent, tupPtr->fiftyPercent,
        tupPtr->unique3, tupPtr->evenOnePercent, tupPtr->oddOnePercent,
        tupPtr->stringu1, tupPtr->stringu2, tupPtr->string4);

    return (1);
}

/************************************************************************
 *									* 
 *  get_result_tuple  							*
 *									* 
 *      get a result tuple back from the database 			*	
 *									* 
 ************************************************************************/

get_result_tuples(dbproc)
   DBPROCESS    *dbproc;
{
   BENCHTUP 	tuple;
    /* bind the program variables to the table columns */

    dbbind(dbproc, 1, INTBIND, (DBINT)0, &tuple.unique1);
    dbbind(dbproc, 2, INTBIND, (DBINT)0, &tuple.unique2);
    dbbind(dbproc, 3, INTBIND, (DBINT)0, &tuple.two);
    dbbind(dbproc, 4, INTBIND, (DBINT)0, &tuple.four);
    dbbind(dbproc, 5, INTBIND, (DBINT)0, &tuple.ten);
    dbbind(dbproc, 6, INTBIND, (DBINT)0, &tuple.twenty);
    dbbind(dbproc, 7, INTBIND, (DBINT)0, &tuple.onePercent);
    dbbind(dbproc, 8, INTBIND, (DBINT)0, &tuple.tenPercent);
    dbbind(dbproc, 9, INTBIND, (DBINT)0, &tuple.twentyPercent);
    dbbind(dbproc, 10, INTBIND, (DBINT)0, &tuple.fiftyPercent);
    dbbind(dbproc, 11, INTBIND, (DBINT)0, &tuple.unique3);
    dbbind(dbproc, 12, INTBIND, (DBINT)0, &tuple.evenOnePercent);
    dbbind(dbproc, 13, INTBIND, (DBINT)0, &tuple.oddOnePercent);
    dbbind(dbproc, 14, CHARBIND, 52, &tuple.stringu1[0]);
    dbbind(dbproc, 15, CHARBIND, 52, &tuple.stringu2[0]);
    dbbind(dbproc, 16, CHARBIND, 52, &tuple.string4[0]);

    while (dbnextrow(dbproc) != NO_MORE_ROWS)
    {
	print_tuple(&tuple);
    }
}


/************************************************************************
 *									* 
 *  print_summary 							*
 *									* 
 *      Print the summary timing info for a query suite			*	
 *									* 
 ************************************************************************/


print_summary(suite_name,db_name,total)
	char	*suite_name;
	char    *db_name;
	int	total;

{
    long clock;

    fprintf (stderr,"============================================================\n");
    fprintf (stderr,"WISCONSIN BENCHMARK - %s\n",suite_name);
    fprintf(stderr,"Total Elapsed Time for Queries: %d.%3.3d Seconds\n",total/1000, total%1000);
    fprintf (stderr,"Name of database: %s\n", db_name);

    /* print current time and date */
    clock = time((long *) 0);
    fprintf(stderr,"Benchmark execution completed: %s\n", ctime(&clock));
    printf ("\n");
}

/*************************************************************************
* open_db                                                                 *
*                                                                         *
*   Open the database                                                     *
*                                                                         *
***************************************************************************/
DBPROCESS*  open_db (dbname, username, passwd, enableBulkCopy)
    char        *dbname, *username, *passwd;
    int	enableBulkCopy;
{
    /* sybase specific variables */
    DBPROCESS       *dbproc;
    RETCODE          return_code;
    char	     temp[500];

    /* next initialize the Sybase DB-Library facility */
    if (dbinit() == FAIL) exit(ERREXIT);

    /* 
       Install the user-supplied error-handling and message-handling routines. 
       They are defined at the bottom of this source file but are stolen
       from the Sybase examples
    */
    dberrhandle(err_handler);
    dbmsghandle(msg_handler);

    login = dblogin();
    DBSETLUSER(login, username);
    DBSETLPWD(login, passwd);
    DBSETLAPP(login, dbname);

    if (enableBulkCopy)
    {
        /* Enable bulk copy for this connection. */
        BCP_SETL(login, TRUE);
    }

    /*
      Get a connection to the database.
    */
    if ((dbproc = dbopen(login, NULL)) == (DBPROCESS *) NULL)
    {
	fprintf(stderr, "Can't connect to server.\n");
	exit(ERREXIT);
    }
    if (VERBOSE) printf("connection to server successful\n");
    /* Turn on option to allow bulk copy */
    if (enableBulkCopy)
    {
	dbuse(dbproc, "master");
        sprintf(temp, "execute sp_dboption %s, 'bulk', true", dbname);
	dbcmd(dbproc, temp);
        dbsqlexec(dbproc);
	while (dbresults(dbproc) != NO_MORE_RESULTS) continue;
        dbuse(dbproc, dbname);
	dbcmd(dbproc, "checkpoint");
        dbsqlexec(dbproc);
        while (dbresults(dbproc) != NO_MORE_RESULTS) continue;
        if (VERBOSE) printf("turned on bulk copy option\n");
    }
    dbuse(dbproc, dbname);
    return (dbproc);
}
 
/**************************************************************************
* close_db                                                                *
*                                                                         *
*   Close the database                                                    *
*                                                                         *
***************************************************************************/
 
 
closedb(dbproc)
   DBPROCESS    *dbproc;
{
    if (VERBOSE) printf("closing database\n");
    if (VERBOSE) printf("first do a checkpoint to truncate the log\n");
    dbcmd(dbproc, "checkpoint");
    dbsqlexec(dbproc);
    while (dbresults(dbproc) != NO_MORE_RESULTS) continue;
    dbexit();
}


int err_handler(dbproc, severity, dberr, oserr, dberrstr, oserrstr)
DBPROCESS       *dbproc;
int             severity;
int             dberr;
int             oserr;
char            *dberrstr;
char            *oserrstr;
{
	if ((dbproc == NULL) || (DBDEAD(dbproc)))
		return(INT_EXIT);
	else 
	{
		fprintf (ERR_CH, "DB-Library error:\n\t%s\n", dberrstr);

		if (oserr != DBNOERR)
			fprintf (ERR_CH, "Operating-system error:\n\t%s\n", oserrstr);

		return(INT_CANCEL);
	}
}

int msg_handler(dbproc, msgno, msgstate, severity, msgtext, 
                srvname, procname, line)

DBPROCESS       *dbproc;
DBINT           msgno;
int             msgstate;
int             severity;
char            *msgtext;
char            *srvname;
char            *procname;
DBUSMALLINT     line;

{
	fprintf (ERR_CH, "Msg %ld, Level %d, State %d\n", 
	        msgno, severity, msgstate);

	if (strlen(srvname) > 0)
		fprintf (ERR_CH, "Server '%s', ", srvname);
	if (strlen(procname) > 0)
		fprintf (ERR_CH, "Procedure '%s', ", procname);
	if (line > 0)
		fprintf (ERR_CH, "Line %d", line);

	fprintf (ERR_CH, "\n\t%s\n", msgtext);

	return(0);
}
